@@ -1,10 +1,10 @@
 namespace eval reverse {
 
-variable is_dingoo [string match *-dingux* $::tcl_platform(osVersion)]
+variable is_dingux [string match dingux "[openmsx_info platform]"]
 variable is_android [string match android "[openmsx_info platform]"]
 
 variable default_auto_enable_reverse
-if {$is_dingoo || $is_android} {
+if {$is_dingux || $is_android} {
 	set default_auto_enable_reverse "off"
 } else {
 	set default_auto_enable_reverse "gui"
