@@ -24,7 +24,7 @@ FirmwareSwitch::FirmwareSwitch(const DeviceConfig& config_)
 		setting.setBoolean(bytebuf != 0);
 	} catch (FileException& e) {
 		config.getCliComm().printWarning(
-			"Couldn't load firmwareswitch status: " + e.getMessage());
+			"Couldn't load firmwareswitch status: ", e.getMessage());
 	}
 }
 
@@ -38,7 +38,7 @@ FirmwareSwitch::~FirmwareSwitch()
 		file.write(&bytebuf, 1);
 	} catch (FileException& e) {
 		config.getCliComm().printWarning(
-			"Couldn't save firmwareswitch status: " + e.getMessage());
+			"Couldn't save firmwareswitch status: ", e.getMessage());
 	}
 }
 
