@@ -2,7 +2,6 @@
 #include "PlugException.hh"
 #include "FileException.hh"
 #include "serialize.hh"
-#include "memory.hh"
 
 namespace openmsx {
 
@@ -49,7 +48,7 @@ void PrinterPortLogger::plugHelper(
 		file = File(logFilenameSetting.getString(),
 		            File::TRUNCATE);
 	} catch (FileException& e) {
-		throw PlugException("Couldn't plug printer logger: " +
+		throw PlugException("Couldn't plug printer logger: ",
 		                    e.getMessage());
 	}
 }
@@ -65,7 +64,7 @@ const std::string& PrinterPortLogger::getName() const
 	return name;
 }
 
-string_ref PrinterPortLogger::getDescription() const
+string_view PrinterPortLogger::getDescription() const
 {
 	return	"Log everything that is sent to the printer port to a "
 		"file. The filename can be set with the "
