@@ -1,7 +1,8 @@
 #include "RenShaTurbo.hh"
 #include "XMLElement.hh"
 #include "Autofire.hh"
-#include "memory.hh"
+#include "MSXException.hh"
+#include <memory>
 
 namespace openmsx {
 
@@ -11,7 +12,12 @@ RenShaTurbo::RenShaTurbo(CommandController& commandController,
 	if (auto* config = machineConfig.findChild("RenShaTurbo")) {
 		int min_ints = config->getChildDataAsInt("min_ints", 47);
 		int max_ints = config->getChildDataAsInt("max_ints", 221);
-		autofire = make_unique<Autofire>(
+		if ((min_ints < 1) || (min_ints > max_ints) || (max_ints > 6000)) {
+			throw MSXException(
+				"Error in RenShaTurbo speed settings: "
+				"1 <= min_ints <= max_ints <= 6000.");
+		}
+		autofire = std::make_unique<Autofire>(
 			commandController, min_ints, max_ints, "renshaturbo");
 	}
 }
