@@ -18,13 +18,13 @@
 #include "Reactor.hh"
 #include "CommandException.hh"
 #include "MemBuffer.hh"
-#include "StringOp.hh"
 #include "serialize.hh"
 #include "serialize_stl.hh"
 #include "xrange.hh"
-#include <functional>
 #include <cassert>
 #include <cmath>
+#include <functional>
+#include <iomanip>
 
 using std::string;
 using std::vector;
@@ -118,7 +118,7 @@ void ReverseManager::ReverseHistory::clear()
 class EndLogEvent final : public StateChange
 {
 public:
-	EndLogEvent() {} // for serialize
+	EndLogEvent() = default; // for serialize
 	explicit EndLogEvent(EmuTime::param time_)
 		: StateChange(time_)
 	{
@@ -252,19 +252,19 @@ void ReverseManager::debugInfo(TclObject& result) const
 {
 	// TODO this is useful during development, but for the end user this
 	// information means nothing. We should remove this later.
-	StringOp::Builder res;
+	string res;
 	size_t totalSize = 0;
 	for (auto& p : history.chunks) {
 		auto& chunk = p.second;
-		res << p.first << ' '
-		    << (chunk.time - EmuTime::zero).toDouble() << ' '
-		    << ((chunk.time - EmuTime::zero).toDouble() / (getCurrentTime() - EmuTime::zero).toDouble()) * 100 << '%'
-		    << " (" << chunk.size << ')'
-		    << " (next event index: " << chunk.eventCount << ")\n";
+		strAppend(res, p.first, ' ',
+		          (chunk.time - EmuTime::zero).toDouble(), ' ',
+		          ((chunk.time - EmuTime::zero).toDouble() / (getCurrentTime() - EmuTime::zero).toDouble()) * 100, "%"
+		          " (", chunk.size, ")"
+		          " (next event index: ", chunk.eventCount, ")\n");
 		totalSize += chunk.size;
 	}
-	res << "total size: " << totalSize << '\n';
-	result.setString(string(res));
+	strAppend(res, "total size: ", totalSize, '\n');
+	result.setString(res);
 }
 
 static void parseGoTo(Interpreter& interp, array_ref<TclObject> tokens,
@@ -681,7 +681,7 @@ void ReverseManager::loadReplay(
 	bool enableViewOnly = false;
 
 	for (size_t i = 2; i < tokens.size(); ++i) {
-		string_ref token = tokens[i].getString();
+		string_view token = tokens[i].getString();
 		if (token == "-viewonly") {
 			enableViewOnly = true;
 		} else if (token == "-goto") {
@@ -724,14 +724,15 @@ void ReverseManager::loadReplay(
 		XmlInputArchive in(filename);
 		in.serialize("replay", replay);
 	} catch (XMLException& e) {
-		throw CommandException("Cannot load replay, bad file format: " + e.getMessage());
+		throw CommandException("Cannot load replay, bad file format: ",
+		                       e.getMessage());
 	} catch (MSXException& e) {
-		throw CommandException("Cannot load replay: " + e.getMessage());
+		throw CommandException("Cannot load replay: ", e.getMessage());
 	}
 
 	// get destination time index
 	auto destination = EmuTime::zero;
-	string_ref where = whereArg ? whereArg->getString() : "begin";
+	string_view where = whereArg ? whereArg->getString() : "begin";
 	if (where == "begin") {
 		destination = EmuTime::zero;
 	} else if (where == "end") {
@@ -893,7 +894,7 @@ unsigned ReverseManager::ReverseHistory::getNextSeqNum(EmuTime::param time) cons
 	}
 	const auto& startTime = begin(chunks)->second.time;
 	double duration = (time - startTime).toDouble();
-	return unsigned(duration / SNAPSHOT_PERIOD + 0.5);
+	return lrint(duration / SNAPSHOT_PERIOD);
 }
 
 void ReverseManager::takeSnapshot(EmuTime::param time)
@@ -1016,7 +1017,7 @@ void ReverseManager::ReverseCmd::execute(array_ref<TclObject> tokens, TclObject&
 	}
 	auto& manager = OUTER(ReverseManager, reverseCmd);
 	auto& interp = getInterpreter();
-	string_ref subcommand = tokens[1].getString();
+	string_view subcommand = tokens[1].getString();
 	if        (subcommand == "start") {
 		manager.start();
 	} else if (subcommand == "stop") {
@@ -1050,7 +1051,7 @@ void ReverseManager::ReverseCmd::execute(array_ref<TclObject> tokens, TclObject&
 			manager.signalStopReplay(manager.getCurrentTime());
 		}
 	} else {
-		throw CommandException("Invalid subcommand: " + subcommand);
+		throw CommandException("Invalid subcommand: ", subcommand);
 	}
 }
 
