@@ -106,9 +106,9 @@ byte SensorKid::getAnalog(byte chi)
 		}
 	} catch (MSXException& e) {
 		getCliComm().printWarning(
-			"Wrong result for callback function \"" +
-			acquireCallback.getSetting().getFullName() +
-			"\": " + e.getMessage());
+			"Wrong result for callback function \"",
+			acquireCallback.getSetting().getFullName(),
+			"\": ", e.getMessage());
 	}
 	return result;
 }
