@@ -43,12 +43,11 @@
 #include "TclObject.hh"
 #include "DynamicClock.hh"
 #include "EmuDuration.hh"
-#include "StringOp.hh"
 #include "serialize.hh"
 #include "unreachable.hh"
-#include "memory.hh"
 #include <algorithm>
 #include <cassert>
+#include <memory>
 
 using std::string;
 using std::vector;
@@ -137,7 +136,7 @@ void CassettePlayer::autoRun()
 		default:
 			UNREACHABLE; // Shouldn't be possible
 	}
-	string command =
+	string command = strCat(
 		"namespace eval ::openmsx {\n"
 		"  variable auto_run_bp\n"
 
@@ -163,18 +162,19 @@ void CassettePlayer::autoRun()
 
 		"  if {[info exists auto_run_bp]} {debug remove_bp $auto_run_bp\n}\n"
 		"  set auto_run_bp [debug set_bp 0xFF07 1 {\n" // H_READ
-		"    openmsx::auto_run_cb " + instr1 + " " + instr2 + "\n"
+		"    openmsx::auto_run_cb ", instr1, ' ', instr2, "\n"
 		"  }]\n"
 
 		// re-trigger hook(s), needed when already booted in BASIC
 		"  type_via_keyboard \'\\r\n"
-		"}";
+		"}");
 	try {
 		motherBoard.getCommandController().executeCommand(command);
 	} catch (CommandException& e) {
 		motherBoard.getMSXCliComm().printWarning(
-			"Error executing loading instruction using command \"" + command + "\" for AutoRun: " +
-			e.getMessage() + "\n Please report a bug.");
+			"Error executing loading instruction using command \"",
+                        command, "\" for AutoRun: ",
+			e.getMessage(), "\n Please report a bug.");
 	}
 }
 
@@ -185,7 +185,7 @@ string CassettePlayer::getStateString() const
 		case RECORD: return "record";
 		case STOP:   return "stop";
 	}
-	UNREACHABLE; return "";
+	UNREACHABLE; return {};
 }
 
 bool CassettePlayer::isRolling() const
@@ -314,19 +314,19 @@ void CassettePlayer::insertTape(const Filename& filename)
 		FilePool& filePool = motherBoard.getReactor().getFilePool();
 		try {
 			// first try WAV
-			playImage = make_unique<WavImage>(filename, filePool);
+			playImage = std::make_unique<WavImage>(filename, filePool);
 		} catch (MSXException& e) {
 			try {
 				// if that fails use CAS
-				playImage = make_unique<CasImage>(
+				playImage = std::make_unique<CasImage>(
 					filename, filePool,
 					motherBoard.getMSXCliComm());
 			} catch (MSXException& e2) {
 				throw MSXException(
-					"Failed to insert WAV image: \"" +
-					e.getMessage() +
-					"\" and also failed to insert CAS image: \"" +
-					e2.getMessage() + '\"');
+					"Failed to insert WAV image: \"",
+					e.getMessage(),
+					"\" and also failed to insert CAS image: \"",
+					e2.getMessage(), '\"');
 			}
 		}
 	} else {
@@ -382,7 +382,7 @@ void CassettePlayer::rewind(EmuTime::param time)
 void CassettePlayer::recordTape(const Filename& filename, EmuTime::param time)
 {
 	removeTape(time); // flush (possible) previous recording
-	recordImage = make_unique<Wav8Writer>(filename, 1, RECORD_FREQ);
+	recordImage = std::make_unique<Wav8Writer>(filename, 1, RECORD_FREQ);
 	tapePos = EmuTime::zero;
 	setState(RECORD, filename, time);
 }
@@ -516,7 +516,7 @@ void CassettePlayer::flushOutput()
 		recordImage->flush(); // update wav header
 	} catch (MSXException& e) {
 		motherBoard.getMSXCliComm().printWarning(
-			"Failed to write to tape: " + e.getMessage());
+			"Failed to write to tape: ", e.getMessage());
 	}
 }
 
@@ -527,7 +527,7 @@ const string& CassettePlayer::getName() const
 	return pluggableName;
 }
 
-string_ref CassettePlayer::getDescription() const
+string_view CassettePlayer::getDescription() const
 {
 	// TODO: this description is not entirely accurate, but it is used
 	// as an identifier for this audio device in e.g. Catapult. We should
@@ -570,7 +570,7 @@ int CassettePlayer::signalEvent(const std::shared_ptr<const Event>& event)
 				playTape(getImageName(), getCurrentTime());
 			} catch (MSXException& e) {
 				motherBoard.getMSXCliComm().printWarning(
-					"Failed to insert tape: " + e.getMessage());
+					"Failed to insert tape: ", e.getMessage());
 			}
 		}
 	}
@@ -633,11 +633,12 @@ void CassettePlayer::TapeCommand::execute(
 		string prefix = "openmsx";
 		string extension = ".wav";
 		string filename = FileOperations::parseCommandFileArgument(
-			(tokens.size() == 3) ? tokens[2].getString() : "",
+			(tokens.size() == 3) ? tokens[2].getString() : string{},
 			directory, prefix, extension);
 		cassettePlayer.recordTape(Filename(filename), time);
-		result.setString("Created new cassette image file: " + filename +
-		                 ", inserted it and set recording mode.");
+		result.setString(strCat(
+                        "Created new cassette image file: ", filename,
+		        ", inserted it and set recording mode."));
 
 	} else if (tokens[1] == "insert" && tokens.size() == 3) {
 		try {
@@ -645,7 +646,7 @@ void CassettePlayer::TapeCommand::execute(
 			Filename filename(tokens[2].getString().str(), userFileContext());
 			cassettePlayer.playTape(filename, time);
 		} catch (MSXException& e) {
-			throw CommandException(e.getMessage());
+			throw CommandException(std::move(e).getMessage());
 		}
 
 	} else if (tokens[1] == "motorcontrol" && tokens.size() == 3) {
@@ -663,8 +664,8 @@ void CassettePlayer::TapeCommand::execute(
 		throw SyntaxError();
 
 	} else if (tokens[1] == "motorcontrol") {
-		result.setString(string("Motor control is ") +
-		                 (cassettePlayer.motorControl ? "on" : "off"));
+		result.setString(strCat("Motor control is ",
+		                 (cassettePlayer.motorControl ? "on" : "off")));
 
 	} else if (tokens[1] == "record") {
 			result.setString("TODO: implement this... (sorry)");
@@ -676,7 +677,7 @@ void CassettePlayer::TapeCommand::execute(
 				cassettePlayer.playTape(
 					cassettePlayer.getImageName(), time);
 			} catch (MSXException& e) {
-				throw CommandException(e.getMessage());
+				throw CommandException(std::move(e).getMessage());
 			}
 		} else if (cassettePlayer.getState() == CassettePlayer::STOP) {
 			throw CommandException("No tape inserted or tape at end!");
@@ -697,7 +698,7 @@ void CassettePlayer::TapeCommand::execute(
 				cassettePlayer.playTape(
 					cassettePlayer.getImageName(), time);
 			} catch (MSXException& e) {
-				throw CommandException(e.getMessage());
+				throw CommandException(std::move(e).getMessage());
 			}
 		}
 		cassettePlayer.rewind(time);
@@ -716,7 +717,7 @@ void CassettePlayer::TapeCommand::execute(
 			Filename filename(tokens[1].getString().str(), userFileContext());
 			cassettePlayer.playTape(filename, time);
 		} catch (MSXException& e) {
-			throw CommandException(e.getMessage());
+			throw CommandException(std::move(e).getMessage());
 		}
 	}
 	//if (!cassettePlayer.getConnector()) {
@@ -850,7 +851,7 @@ void CassettePlayer::serialize(Archive& ar, unsigned version)
 	}
 	if (ar.versionAtLeast(version, 2)) {
 		string oldChecksumStr = oldChecksum.empty()
-		                      ? ""
+		                      ? string{}
 		                      : oldChecksum.toString();
 		ar.serialize("checksum", oldChecksumStr);
 		oldChecksum = oldChecksumStr.empty()
@@ -891,8 +892,8 @@ void CassettePlayer::serialize(Archive& ar, unsigned version)
 			Sha1Sum newChecksum = playImage->getSha1Sum();
 			if (oldChecksum != newChecksum) {
 				motherBoard.getMSXCliComm().printWarning(
-					"The content of the tape " +
-					casImage.getResolved() +
+					"The content of the tape ",
+					casImage.getResolved(),
 					" has changed since the time this "
 					"savestate was created. This might "
 					"result in emulation problems.");
