@@ -17,9 +17,9 @@ void DummyCassetteDevice::setSignal(bool /*output*/, EmuTime::param /*time*/)
 	// do nothing
 }
 
-string_ref DummyCassetteDevice::getDescription() const
+string_view DummyCassetteDevice::getDescription() const
 {
-	return "";
+	return {};
 }
 
 void DummyCassetteDevice::plugHelper(Connector& /*connector*/,
