@@ -7,11 +7,11 @@
 #include "Z80.hh"
 #include "R800.hh"
 #include "TclObject.hh"
-#include "memory.hh"
 #include "outer.hh"
 #include "serialize.hh"
 #include "unreachable.hh"
 #include <cassert>
+#include <memory>
 
 using std::string;
 using std::vector;
@@ -26,18 +26,18 @@ MSXCPU::MSXCPU(MSXMotherBoard& motherboard_)
 	, diHaltCallback(
 		motherboard.getCommandController(), "di_halt_callback",
 		"Tcl proc called when the CPU executed a DI/HALT sequence")
-	, z80(make_unique<CPUCore<Z80TYPE>>(
+	, z80(std::make_unique<CPUCore<Z80TYPE>>(
 		motherboard, "z80", traceSetting,
 		diHaltCallback, EmuTime::zero))
 	, r800(motherboard.isTurboR()
-		? make_unique<CPUCore<R800TYPE>>(
+		? std::make_unique<CPUCore<R800TYPE>>(
 			motherboard, "r800", traceSetting,
 			diHaltCallback, EmuTime::zero)
 		: nullptr)
 	, timeInfo(motherboard.getMachineInfoCommand())
 	, z80FreqInfo(motherboard.getMachineInfoCommand(), "z80_freq", *z80)
 	, r800FreqInfo(r800
-		? make_unique<CPUFreqInfoTopic>(
+		? std::make_unique<CPUFreqInfoTopic>(
 			motherboard.getMachineInfoCommand(), "r800_freq", *r800)
 		: nullptr)
 	, debuggable(motherboard_)
