@@ -163,80 +163,3 @@ void AdhocCliCommParser::parse(char c)
 		break;
 	}
 }
-
-
-
-#if 0
-
-#include <iostream>
-#include <vector>
-
-using namespace std;
-
-template<typename T>
-void print(const T& t)
-{
-	for (auto& i : t) cout << i << std::endl;
-}
-
-void test(const string& stream, const vector<string>& expectedCommands)
-{
-	vector<string> result;
-	AdhocCliCommParser parser([&](const string& cmd) { result.push_back(cmd); });
-	parser.parse(stream.data(), stream.size());
-	if (result != expectedCommands) {
-		cout << "ERROR while parsing " << stream << endl;
-		cout << "Expected to see:" << endl;
-		print(expectedCommands);
-		cout << "but got:" << endl;
-		print(result);
-	} else {
-		cout << "ok" << endl;
-	}
-}
-
-int main()
-{
-	test("<command>foo</command>", {"foo"});
-	test("  <command>foo</command>", {"foo"});
-	test("<command>foo</command>  ", {"foo"});
-	test("  <command>foo</command>  ", {"foo"});
-
-	test("<command>foo</command><command>bar</command>", {"foo", "bar"});
-	test("<command>foo</command>  <command>bar</command>", {"foo", "bar"});
-
-	test("<command>&amp;</command>",  {"&"});
-	test("<command>&apos;</command>", {"'"});
-	test("<command>&quot;</command>", {"\""});
-	test("<command>&lt;</command>",   {"<"});
-	test("<command>&gt;</command>",   {">"});
-	test("<command>&#65;</command>",  {"A"});
-	test("<command>&#x41;</command>", {"A"});
-	test("<command>&lt;command&gt;</command>", {"<command>"});
-
-	test("<openmsx-control> <command>foo</command> </openmsx-control>",
-	     {"foo"});
-	test("<openmsx-control> <unknown>foo</unknown> <command>foo</command>",
-	     {"foo"});
-
-	// errors, but we do recover
-	test("<command>&unknown;</command><command>foo</command>", {"foo"});
-	test("<command>&#fffffff;</command><command>foo</command>", {"foo"});
-	test("<<<<<command>foo</command>", {"foo"});
-	test("<foo></bar><command>foo</command>", {"foo"});
-	test("</command><command>foo</command>", {"foo"});
-	test("<command>foo</foobar><command>foo</command>", {"foo"});
-
-	// This was not accepted before, but now is
-	test("<openmsx-control></openmsx-control><command>foo</command>", {"foo"});
-
-	// This was accepted before but now isn't
-	test("<command value=\"3\">foo</command>",
-	     {}); // should be "foo"
-	test("<command><bla></bla>foo</command>",
-	     {}); // should be "foo"
-	test("<command/>",
-	     {}); // sort-of OK, was before an empty command, now no command
-}
-
-#endif
