@@ -7,27 +7,27 @@ const char* const CliComm::levelStr[CliComm::NUM_LEVELS] = {
 };
 
 const char* const CliComm::updateStr[CliComm::NUM_UPDATES] = {
-	"led", "setting", "setting-info", "hardware", "plug", "unplug",
+	"led", "setting", "setting-info", "hardware", "plug",
 	"media", "status", "extension", "sounddevice", "connector"
 };
 
 
-void CliComm::printInfo(string_ref message)
+void CliComm::printInfo(string_view message)
 {
 	log(INFO, message);
 }
 
-void CliComm::printWarning(string_ref message)
+void CliComm::printWarning(string_view message)
 {
 	log(WARNING, message);
 }
 
-void CliComm::printError(string_ref message)
+void CliComm::printError(string_view message)
 {
 	log(LOGLEVEL_ERROR, message);
 }
 
-void CliComm::printProgress(string_ref message)
+void CliComm::printProgress(string_view message)
 {
 	log(PROGRESS, message);
 }
