@@ -10,12 +10,12 @@ MSXCliComm::MSXCliComm(MSXMotherBoard& motherBoard_, GlobalCliComm& cliComm_)
 {
 }
 
-void MSXCliComm::log(LogLevel level, string_ref message)
+void MSXCliComm::log(LogLevel level, string_view message)
 {
 	cliComm.log(level, message);
 }
 
-void MSXCliComm::update(UpdateType type, string_ref name, string_ref value)
+void MSXCliComm::update(UpdateType type, string_view name, string_view value)
 {
 	assert(type < NUM_UPDATES);
 	auto it = prevValues[type].find(name);
