@@ -11,7 +11,7 @@ MessageCommand::MessageCommand(CommandController& controller)
 {
 }
 
-static CliComm::LogLevel getLevel(string_ref level)
+static CliComm::LogLevel getLevel(string_view level)
 {
 	auto levels = CliComm::getLevelStrings();
 	for (auto i : xrange(levels.size())) {
@@ -19,7 +19,7 @@ static CliComm::LogLevel getLevel(string_ref level)
 			return static_cast<CliComm::LogLevel>(i);
 		}
 	}
-	throw CommandException("Unknown level string: " + level);
+	throw CommandException("Unknown level string: ", level);
 }
 
 void MessageCommand::execute(array_ref<TclObject> tokens, TclObject& /*result*/)
