@@ -38,11 +38,11 @@ void RawTrack::addIdam(unsigned idx)
 	idam.push_back(idx);
 }
 
-void RawTrack::write(int idx, byte val, bool addIdam)
+void RawTrack::write(int idx, byte val, bool setIdam)
 {
 	unsigned i2 = wrapIndex(idx);
 	auto it = lower_bound(begin(idam), end(idam), i2);
-	if (addIdam) {
+	if (setIdam) {
 		// add idam (if not already present)
 		if ((it == end(idam)) || (*it != i2)) {
 			idam.insert(it, i2);
@@ -274,7 +274,7 @@ template<typename Archive>
 void RawTrack::serialize(Archive& ar, unsigned version)
 {
 	ar.serialize("idam", idam);
-	unsigned len = unsigned(data.size());
+	auto len = unsigned(data.size());
 	if (ar.versionAtLeast(version, 2)) {
 		ar.serialize("trackLength", len);
 	} else {
