@@ -9,7 +9,7 @@
 #include "GlobalSettings.hh"
 #include "MSXException.hh"
 #include "serialize.hh"
-#include "memory.hh"
+#include <memory>
 
 using std::string;
 
@@ -42,16 +42,20 @@ RealDrive::RealDrive(MSXMotherBoard& motherBoard_, EmuDuration::param motorTimeo
 	string driveName = "diskX"; driveName[4] = char('a' + i);
 
 	if (motherBoard.getCommandController().hasCommand(driveName)) {
-		throw MSXException("Duplicated drive name: " + driveName);
+		throw MSXException("Duplicated drive name: ", driveName);
 	}
 	motherBoard.getMSXCliComm().update(CliComm::HARDWARE, driveName, "add");
-	changer = make_unique<DiskChanger>(motherBoard, driveName, true, doubleSizedDrive,
-	                                   [this]() { invalidateTrack(); });
+	changer = std::make_unique<DiskChanger>(motherBoard, driveName, true, doubleSizedDrive,
+	                                        [this]() { invalidateTrack(); });
 }
 
 RealDrive::~RealDrive()
 {
-	flushTrack();
+	try {
+		flushTrack();
+	} catch (MSXException&) {
+		// ignore
+	}
 	doSetMotor(false, getCurrentTime()); // to send LED event
 
 	const auto& driveName = changer->getDriveName();
@@ -261,7 +265,11 @@ EmuTime RealDrive::getTimeTillIndexPulse(EmuTime::param time, int count)
 
 void RealDrive::invalidateTrack()
 {
-	flushTrack();
+	try {
+		flushTrack();
+	} catch (MSXException&) {
+		// ignore
+	}
 	trackValid = false;
 }
 
