@@ -4,7 +4,7 @@
 #include "DiskExceptions.hh"
 #include "sha1.hh"
 #include "xrange.hh"
-#include "memory.hh"
+#include <memory>
 
 namespace openmsx {
 
@@ -16,7 +16,7 @@ const size_t SectorAccessibleDisk::SECTOR_SIZE;
 #endif
 
 SectorAccessibleDisk::SectorAccessibleDisk()
-	: patch(make_unique<EmptyDiskPatch>(*this))
+	: patch(std::make_unique<EmptyDiskPatch>(*this))
 	, forcedWriteProtect(false)
 	, peekMode(false)
 {
@@ -38,14 +38,14 @@ void SectorAccessibleDisk::readSector(size_t sector, SectorBuffer& buf)
 		// in the end this calls readSectorImpl()
 		patch->copyBlock(sector * sizeof(buf), buf.raw, sizeof(buf));
 	} catch (MSXException& e) {
-		throw DiskIOErrorException("Disk I/O error: " + e.getMessage());
+		throw DiskIOErrorException("Disk I/O error: ", e.getMessage());
 	}
 }
 
 void SectorAccessibleDisk::writeSector(size_t sector, const SectorBuffer& buf)
 {
 	if (isWriteProtected()) {
-		throw WriteProtectedException("");
+		throw WriteProtectedException();
 	}
 	if (!isDummyDisk() && (getNbSectors() <= sector)) {
 		throw NoSuchSectorException("No such sector");
@@ -53,7 +53,7 @@ void SectorAccessibleDisk::writeSector(size_t sector, const SectorBuffer& buf)
 	try {
 		writeSectorImpl(sector, buf);
 	} catch (MSXException& e) {
-		throw DiskIOErrorException("Disk I/O error: " + e.getMessage());
+		throw DiskIOErrorException("Disk I/O error: ", e.getMessage());
 	}
 	flushCaches();
 }
@@ -65,7 +65,7 @@ size_t SectorAccessibleDisk::getNbSectors() const
 
 void SectorAccessibleDisk::applyPatch(Filename patchFile)
 {
-	patch = make_unique<IPSPatch>(std::move(patchFile), std::move(patch));
+	patch = std::make_unique<IPSPatch>(std::move(patchFile), std::move(patch));
 }
 
 std::vector<Filename> SectorAccessibleDisk::getPatches() const
