@@ -62,6 +62,7 @@ WD2793::WD2793(Scheduler& scheduler_, DiskDrive& drive_, CliComm& cliComm_,
 	dataRegWritten = false;
 	lastWasA1 = false;
 	lastWasCRC = false;
+	commandReg = 0;
 	setDrqRate(RawTrack::STANDARD_SIZE);
 
 	reset(time);
@@ -631,7 +632,7 @@ void WD2793::startReadSector(EmuTime::param time)
 		crc.init<0xA1, 0xA1, 0xA1, 0xFB>();
 	}
 	unsigned trackLength = drive.getTrackLength();
-	int tmp = sectorInfo.dataIdx >= sectorInfo.addrIdx;
+	int tmp = sectorInfo.dataIdx - sectorInfo.addrIdx;
 	unsigned gapLength = (tmp >= 0) ? tmp : (tmp + trackLength);
 	assert(gapLength < trackLength);
 	drqTime.reset(time);
