@@ -4,8 +4,9 @@
 
 namespace openmsx {
 
-WD2793BasedFDC::WD2793BasedFDC(const DeviceConfig& config, const std::string& romId)
-	: MSXFDC(config, romId)
+WD2793BasedFDC::WD2793BasedFDC(const DeviceConfig& config, const std::string& romId,
+                               bool needROM)
+	: MSXFDC(config, romId, needROM)
 	, multiplexer(reinterpret_cast<DiskDrive**>(drives))
 	, controller(
 		getScheduler(), multiplexer, getCliComm(), getCurrentTime(),
