@@ -20,10 +20,10 @@ const string USER_DATA    = "{{USER_DATA}}";
 const string SYSTEM_DATA  = "{{SYSTEM_DATA}}";
 
 
-static string subst(string_ref path, string_ref before, string_ref after)
+static string subst(string_view path, string_view before, string_view after)
 {
 	assert(path.starts_with(before));
-	return after + path.substr(before.size());
+	return strCat(after, path.substr(before.size()));
 }
 
 static vector<string> getPathsHelper(const vector<string>& input)
@@ -49,7 +49,7 @@ static vector<string> getPathsHelper(const vector<string>& input)
 }
 
 static string resolveHelper(const vector<string>& pathList,
-                            string_ref filename)
+                            string_view filename)
 {
 	string filepath = FileOperations::expandCurrentDirFromDrive(filename);
 	filepath = FileOperations::expandTilde(filepath);
@@ -66,7 +66,7 @@ static string resolveHelper(const vector<string>& pathList,
 		}
 	}
 	// not found in any path
-	throw FileException(filename + " not found in this context");
+	throw FileException(filename, " not found in this context");
 }
 
 FileContext::FileContext(vector<string>&& paths_, vector<string>&& savePaths_)
@@ -74,7 +74,7 @@ FileContext::FileContext(vector<string>&& paths_, vector<string>&& savePaths_)
 {
 }
 
-const string FileContext::resolve(string_ref filename) const
+const string FileContext::resolve(string_view filename) const
 {
 	vector<string> pathList = getPathsHelper(paths);
 	string result = resolveHelper(pathList, filename);
@@ -82,7 +82,7 @@ const string FileContext::resolve(string_ref filename) const
 	return result;
 }
 
-const string FileContext::resolveCreate(string_ref filename) const
+const string FileContext::resolveCreate(string_view filename) const
 {
 	string result;
 	vector<string> pathList = getPathsHelper(savePaths);
@@ -139,7 +139,7 @@ static string backSubstSymbols(const string& path)
 	return path;
 }
 
-FileContext configFileContext(string_ref path, string_ref hwDescr, string_ref userName)
+FileContext configFileContext(string_view path, string_view hwDescr, string_view userName)
 {
 	return { { backSubstSymbols(FileOperations::expandTilde(path)) },
 	         { FileOperations::join(
@@ -158,25 +158,25 @@ FileContext preferSystemFileContext()
 	         {} };
 }
 
-FileContext userFileContext(string_ref savePath)
+FileContext userFileContext(string_view savePath)
 {
 	vector<string> savePaths;
 	if (!savePath.empty()) {
 		savePaths = { FileOperations::join(
 		                 USER_OPENMSX, "persistent", savePath) };
 	}
-	return { { "", USER_DIRS }, std::move(savePaths) };
+	return { { string{}, USER_DIRS }, std::move(savePaths) };
 }
 
-FileContext userDataFileContext(string_ref subDir)
+FileContext userDataFileContext(string_view subDir)
 {
-	return { { "", USER_OPENMSX + '/' + subDir },
+	return { { string{}, strCat(USER_OPENMSX, '/', subDir) },
 	         {} };
 }
 
 FileContext currentDirFileContext()
 {
-	return { { "" },
+	return { { string{} },
 	         {} };
 }
 
