@@ -4,12 +4,12 @@
 
 namespace openmsx {
 
-const byte ASCII_FLAG   = 0x01; // bit 0 set: file probably ascii text
-const byte HEAD_CRC     = 0x02; // bit 1 set: header CRC present
-const byte EXTRA_FIELD  = 0x04; // bit 2 set: extra field present
-const byte ORIG_NAME    = 0x08; // bit 3 set: original file name present
-const byte COMMENT      = 0x10; // bit 4 set: file comment present
-const byte RESERVED     = 0xE0; // bits 5..7: reserved
+static const byte ASCII_FLAG  = 0x01; // bit 0 set: file probably ascii text
+static const byte HEAD_CRC    = 0x02; // bit 1 set: header CRC present
+static const byte EXTRA_FIELD = 0x04; // bit 2 set: extra field present
+static const byte ORIG_NAME   = 0x08; // bit 3 set: original file name present
+static const byte COMMENT     = 0x10; // bit 4 set: file comment present
+static const byte RESERVED    = 0xE0; // bits 5..7: reserved
 
 
 GZFileAdapter::GZFileAdapter(std::unique_ptr<FileBase> file_)
