@@ -44,56 +44,17 @@ const byte* BeerIDE::getReadCacheLine(word start) const
 
 byte BeerIDE::readIO(word port, EmuTime::param time)
 {
-        switch (port & 0x03) {
-        case 0:
-                return i8255.readPortA(time);
-        case 1:
-                return i8255.readPortB(time);
-        case 2:
-                return i8255.readPortC(time);
-        case 3:
-                return i8255.readControlPort(time);
-        default: // unreachable, avoid warning
-                UNREACHABLE;
-                return 0;
-        }
+	return i8255.read(port & 0x03, time);
 }
 
 byte BeerIDE::peekIO(word port, EmuTime::param time) const
 {
-        switch (port & 0x03) {
-        case 0:
-                return i8255.peekPortA(time);
-        case 1:
-                return i8255.peekPortB(time);
-        case 2:
-                return i8255.peekPortC(time);
-        case 3:
-                return i8255.readControlPort(time);
-        default: // unreachable, avoid warning
-                UNREACHABLE;
-                return 0;
-        }
+	return i8255.peek(port & 0x03, time);
 }
 
 void BeerIDE::writeIO(word port, byte value, EmuTime::param time)
 {
-        switch (port & 0x03) {
-        case 0:
-                i8255.writePortA(value, time);
-                break;
-        case 1:
-                i8255.writePortB(value, time);
-                break;
-        case 2:
-                i8255.writePortC(value, time);
-                break;
-        case 3:
-                i8255.writeControlPort(value, time);
-                break;
-        default:
-                UNREACHABLE;
-        }
+	i8255.write(port & 0x03, value, time);
 }
 
 // I8255Interface
