@@ -9,10 +9,10 @@
 #include "CliComm.hh"
 #include "endian.hh"
 #include "serialize.hh"
-#include "memory.hh"
 #include <algorithm>
 #include <cassert>
 #include <cstdio>
+#include <memory>
 
 using std::string;
 using std::vector;
@@ -49,7 +49,7 @@ IDECDROM::IDECDROM(const DeviceConfig& config)
 	}
 	name[2] = char('a' + id);
 	(*cdInUse)[id] = true;
-	cdxCommand = make_unique<CDXCommand>(
+	cdxCommand = std::make_unique<CDXCommand>(
 		getMotherBoard().getCommandController(),
 		getMotherBoard().getStateChangeDistributor(),
 		getMotherBoard().getScheduler(), *this);
@@ -59,6 +59,10 @@ IDECDROM::IDECDROM(const DeviceConfig& config)
 	remMedStatNotifEnabled = false;
 	mediaChanged = false;
 
+	byteCountLimit = 0; // avoid UMR in serialize()
+	transferOffset = 0;
+	readSectorData = false;
+
 	getMotherBoard().getMSXCliComm().update(CliComm::HARDWARE, name, "add");
 }
 
@@ -308,7 +312,7 @@ void IDECDROM::eject()
 	file.close();
 	mediaChanged = true;
 	senseKey = 0x06 << 16; // unit attention (medium changed)
-	getMotherBoard().getMSXCliComm().update(CliComm::MEDIA, name, "");
+	getMotherBoard().getMSXCliComm().update(CliComm::MEDIA, name, {});
 }
 
 void IDECDROM::insert(const string& filename)
@@ -337,7 +341,7 @@ void CDXCommand::execute(array_ref<TclObject> tokens, TclObject& result,
 	if (tokens.size() == 1) {
 		auto& file = cd.file;
 		result.addListElement(cd.name + ':');
-		result.addListElement(file.is_open() ? file.getURL() : "");
+		result.addListElement(file.is_open() ? file.getURL() : string{});
 		if (!file.is_open()) result.addListElement("empty");
 	} else if ((tokens.size() == 2) &&
 	           ((tokens[1] == "eject") || (tokens[1] == "-eject"))) {
@@ -365,8 +369,8 @@ void CDXCommand::execute(array_ref<TclObject> tokens, TclObject& result,
 			cd.insert(filename);
 			// return filename; // Note: the diskX command doesn't do this either, so this has not been converted to TclObject style here
 		} catch (FileException& e) {
-			throw CommandException("Can't change cd image: " +
-					e.getMessage());
+			throw CommandException("Can't change cd image: ",
+			                       e.getMessage());
 		}
 	} else {
 		throw CommandException("Too many or wrong arguments.");
@@ -375,10 +379,11 @@ void CDXCommand::execute(array_ref<TclObject> tokens, TclObject& result,
 
 string CDXCommand::help(const vector<string>& /*tokens*/) const
 {
-	return cd.name + "                   : display the cd image for this CDROM drive\n" +
-	       cd.name + " eject             : eject the cd image from this CDROM drive\n" +
-	       cd.name + " insert <filename> : change the cd image for this CDROM drive\n" +
-	       cd.name + " <filename>        : change the cd image for this CDROM drive\n";
+	return strCat(
+		cd.name, "                   : display the cd image for this CDROM drive\n",
+		cd.name, " eject             : eject the cd image from this CDROM drive\n",
+		cd.name, " insert <filename> : change the cd image for this CDROM drive\n",
+		cd.name, " <filename>        : change the cd image for this CDROM drive\n");
 }
 
 void CDXCommand::tabCompletion(vector<string>& tokens) const
@@ -393,7 +398,7 @@ void IDECDROM::serialize(Archive& ar, unsigned /*version*/)
 {
 	ar.template serializeBase<AbstractIDEDevice>(*this);
 
-	string filename = file.is_open() ? file.getURL() : "";
+	string filename = file.is_open() ? file.getURL() : string{};
 	ar.serialize("filename", filename);
 	if (ar.isLoader()) {
 		// re-insert CDROM before restoring 'mediaChanged', 'senseKey'
