@@ -12,9 +12,9 @@ void DummyJoystick::write(byte /*value*/, EmuTime::param /*time*/)
 	// do nothing
 }
 
-string_ref DummyJoystick::getDescription() const
+string_view DummyJoystick::getDescription() const
 {
-	return "";
+	return {};
 }
 
 void DummyJoystick::plugHelper(Connector& /*connector*/,
