@@ -8,9 +8,9 @@
 #include "checked_cast.hh"
 #include "serialize.hh"
 #include "serialize_meta.hh"
-#include "memory.hh"
 #include "unreachable.hh"
 #include "build-info.hh"
+#include <memory>
 
 using std::string;
 using std::shared_ptr;
@@ -43,7 +43,7 @@ void JoyMega::registerAll(MSXEventDistributor& eventDistributor,
 			// practice.
 			if (InputEventGenerator::joystickNumButtons(joystick) != 0) {
 				controller.registerPluggable(
-					make_unique<JoyMega>(
+					std::make_unique<JoyMega>(
 						eventDistributor,
 						stateChangeDistributor,
 						joystick));
@@ -56,7 +56,7 @@ void JoyMega::registerAll(MSXEventDistributor& eventDistributor,
 class JoyMegaState final : public StateChange
 {
 public:
-	JoyMegaState() {} // for serialize
+	JoyMegaState() = default; // for serialize
 	JoyMegaState(EmuTime::param time_, unsigned joyNum_,
 	             unsigned press_, unsigned release_)
 		: StateChange(time_)
@@ -115,7 +115,7 @@ const string& JoyMega::getName() const
 	return name;
 }
 
-string_ref JoyMega::getDescription() const
+string_view JoyMega::getDescription() const
 {
 	return desc;
 }
