@@ -13,9 +13,9 @@
 #include "CommandException.hh"
 #include "serialize.hh"
 #include "serialize_meta.hh"
-#include "memory.hh"
 #include "xrange.hh"
 #include "build-info.hh"
+#include <memory>
 
 using std::string;
 using std::shared_ptr;
@@ -46,7 +46,7 @@ void Joystick::registerAll(MSXEventDistributor& eventDistributor,
 			// practice.
 			if (InputEventGenerator::joystickNumButtons(joystick) != 0) {
 				controller.registerPluggable(
-					make_unique<Joystick>(
+					std::make_unique<Joystick>(
 						eventDistributor,
 						stateChangeDistributor,
 						commandController,
@@ -62,7 +62,7 @@ void Joystick::registerAll(MSXEventDistributor& eventDistributor,
 class JoyState final : public StateChange
 {
 public:
-	JoyState() {} // for serialize
+	JoyState() = default; // for serialize
 	JoyState(EmuTime::param time_, unsigned joyNum_, byte press_, byte release_)
 		: StateChange(time_)
 		, joyNum(joyNum_), press(press_), release(release_)
@@ -96,7 +96,7 @@ static void checkJoystickConfig(Interpreter& interp, TclObject& newValue)
 		throw CommandException("Need an even number of elements");
 	}
 	for (unsigned i = 0; i < n; i += 2) {
-		string_ref key  = newValue.getListIndex(interp, i + 0).getString();
+		string_view key  = newValue.getListIndex(interp, i + 0).getString();
 		TclObject value = newValue.getListIndex(interp, i + 1);
 		if ((key != "A"   ) && (key != "B"    ) &&
 		    (key != "LEFT") && (key != "RIGHT") &&
@@ -106,7 +106,7 @@ static void checkJoystickConfig(Interpreter& interp, TclObject& newValue)
 				"'A', 'B', 'LEFT', 'RIGHT', 'UP', 'DOWN'.");
 		}
 		for (auto j : xrange(value.getListLength(interp))) {
-			string_ref host = value.getListIndex(interp, j).getString();
+			string_view host = value.getListIndex(interp, j).getString();
 			if (!host.starts_with("button") &&
 			    !host.starts_with("+axis") &&
 			    !host.starts_with("-axis") &&
@@ -125,7 +125,7 @@ static void checkJoystickConfig(Interpreter& interp, TclObject& newValue)
 
 static string getJoystickName(unsigned joyNum)
 {
-	return string("joystick") + char('1' + joyNum);
+	return strCat("joystick", char('1' + joyNum));
 }
 
 static TclObject getConfigValue(SDL_Joystick* joystick)
@@ -137,7 +137,7 @@ static TclObject getConfigValue(SDL_Joystick* joystick)
 	value.addListElement("DOWN" ); value.addListElement("+axis1");
 	TclObject listA, listB;
 	for (auto i : xrange(InputEventGenerator::joystickNumButtons(joystick))) {
-		string button = "button" + StringOp::toString(i);
+		string button = strCat("button", i);
 		if (i & 1) {
 			listB.addListElement(button);
 		} else {
@@ -191,7 +191,7 @@ const string& Joystick::getName() const
 	return name;
 }
 
-string_ref Joystick::getDescription() const
+string_view Joystick::getDescription() const
 {
 	return desc;
 }
@@ -248,7 +248,7 @@ byte Joystick::calcState()
 }
 
 bool Joystick::getState(Interpreter& interp, const TclObject& dict,
-                        string_ref key, int threshold)
+                        string_view key, int threshold)
 {
 	try {
 		const auto& list = dict.getDictValue(interp, TclObject(key));
