@@ -3,15 +3,14 @@
 #include "yuv2rgb.hh"
 #include "likely.hh"
 #include "CliComm.hh"
-#include "StringOp.hh"
 #include "MemoryOps.hh"
-#include "memory.hh"
 #include "stl.hh"
 #include "stringsp.hh" // for strncasecmp
 #include <algorithm>
 #include <cstring> // for memcpy, memcmp
 #include <cstdlib> // for atoi
 #include <cctype> // for isspace
+#include <memory>
 
 // TODO
 // - Improve error handling
@@ -362,7 +361,7 @@ void OggReader::readVorbis(ogg_packet* packet)
 	while (pos < decoded)  {
 		// Find memory to copy PCM into
 		if (recycleAudioList.empty()) {
-			auto audio = make_unique<AudioFragment>();
+			auto audio = std::make_unique<AudioFragment>();
 			audio->length = 0;
 			recycleAudioList.push_back(std::move(audio));
 		}
@@ -408,12 +407,9 @@ void OggReader::readVorbis(ogg_packet* packet)
 			vorbisFoundPosition();
 		} else {
 			if (vorbisPos != size_t(packet->granulepos)) {
-				cli.printWarning("vorbis audio out of sync, "
-					"expected " +
-					StringOp::toString(vorbisPos) +
-					", got " +
-					StringOp::toString(packet->granulepos));
-
+				cli.printWarning(
+                                        "vorbis audio out of sync, expected ",
+					vorbisPos, ", got ", packet->granulepos);
 				vorbisPos = packet->granulepos;
 			}
 		}
@@ -429,7 +425,7 @@ size_t OggReader::frameNo(ogg_packet* packet)
 		return size_t(-1);
 	}
 
-	size_t intra = packet->granulepos & ((1 << granuleShift) - 1);
+	size_t intra = packet->granulepos & ((size_t(1) << granuleShift) - 1);
 	size_t key = packet->granulepos >> granuleShift;
 	return key + intra;
 }
@@ -550,8 +546,7 @@ void OggReader::readTheora(ogg_packet* packet)
 	case 0:
 		break;
 	default:
-		cli.printWarning("Theora error: unknown error " +
-					StringOp::toString(rc));
+		cli.printWarning("Theora error: unknown error ", rc);
 		break;
 	}
 
@@ -573,7 +568,7 @@ void OggReader::readTheora(ogg_packet* packet)
 
 	std::unique_ptr<Frame> frame;
 	if (recycleFrameList.empty()) {
-		frame = make_unique<Frame>(yuv);
+		frame = std::make_unique<Frame>(yuv);
 	} else {
 		frame = std::move(recycleFrameList.back());
 		recycleFrameList.pop_back();
@@ -640,9 +635,9 @@ void OggReader::getFrameNo(RawFrame& rawFrame, size_t frameno)
 		if (!frameList.empty() && frameList[0]->no > frameno) {
 			// we're missing frames!
 			frame = frameList[0].get();
-			cli.printWarning("Cannot find frame " +
-				StringOp::toString(frameno) + " using " +
-				StringOp::toString(frame->no) + " instead");
+			cli.printWarning(
+                                "Cannot find frame ", frameno, " using ",
+			        frame->no, " instead");
 			break;
 		}
 
@@ -661,11 +656,10 @@ void OggReader::getFrameNo(RawFrame& rawFrame, size_t frameno)
 		}
 
 		// Sanity check, should not happen
-		if (frameList.size() > (2u << granuleShift)) {
+		if (frameList.size() > (size_t(2) << granuleShift)) {
 			// We've got more than twice as many frames
 			// as the maximum distance between key frames.
-			cli.printWarning("Cannot find frame " +
-				StringOp::toString(frameno));
+			cli.printWarning("Cannot find frame ", frameno);
 			return;
 		}
 
@@ -765,8 +759,8 @@ bool OggReader::nextPacket()
 				cli.printWarning("Failed to submit theora page");
 			}
 		} else if (serial != skeletonSerial) {
-			cli.printWarning("Unexpected stream with serial " +
-				StringOp::toString(serial) + " in ogg file");
+			cli.printWarning("Unexpected stream with serial ",
+			                 serial, " in ogg file");
 		}
 	}
 }
