@@ -1,11 +1,9 @@
 #include "RomDatabase.hh"
-#include "CommandException.hh"
-#include "TclObject.hh"
 #include "FileContext.hh"
 #include "File.hh"
 #include "FileOperations.hh"
-#include "GlobalCommandController.hh"
 #include "CliComm.hh"
+#include "MSXException.hh"
 #include "StringOp.hh"
 #include "String32.hh"
 #include "hash_map.hh"
@@ -14,6 +12,7 @@
 #include "unreachable.hh"
 #include "stl.hh"
 #include "xxhash.hh"
+#include <cassert>
 #include <stdexcept>
 
 using std::string;
@@ -39,16 +38,16 @@ public:
 	}
 
 	// rapidsax handler interface
-	void start(string_ref name);
-	void attribute(string_ref name, string_ref value);
-	void text(string_ref txt);
+	void start(string_view tag);
+	void attribute(string_view name, string_view value);
+	void text(string_view txt);
 	void stop();
-	void doctype(string_ref txt);
+	void doctype(string_view txt);
 
-	string_ref getSystemID() const { return systemID; }
+	string_view getSystemID() const { return systemID; }
 
 private:
-	String32 cIndex(string_ref str);
+	String32 cIndex(string_view str);
 	void addEntries();
 	void addAllEntries();
 
@@ -86,12 +85,12 @@ private:
 	CliComm& cliComm;
 	char* bufStart;
 
-	string_ref systemID;
-	string_ref type;
-	string_ref startVal;
+	string_view systemID;
+	string_view type;
+	string_view startVal;
 
 	vector<Dump> dumps;
-	string_ref system;
+	string_view system;
 	String32 title;
 	String32 company;
 	String32 year;
@@ -103,13 +102,14 @@ private:
 	size_t initialSize;
 };
 
-void DBParser::start(string_ref tag)
+void DBParser::start(string_view tag)
 {
 	if (unknownLevel) {
 		++unknownLevel;
 		return;
 	}
 
+	assert(!tag.empty()); // rapidsax will reject empty tags
 	switch (state) {
 	case BEGIN:
 		if (tag == "softwaredb") {
@@ -259,7 +259,7 @@ void DBParser::start(string_ref tag)
 		break;
 
 	case END:
-		throw MSXException("Unexpected opening tag: " + tag);
+		throw MSXException("Unexpected opening tag: ", tag);
 
 	default:
 		UNREACHABLE;
@@ -268,7 +268,7 @@ void DBParser::start(string_ref tag)
 	++unknownLevel;
 }
 
-void DBParser::attribute(string_ref name, string_ref value)
+void DBParser::attribute(string_view name, string_view value)
 {
 	if (unknownLevel) return;
 
@@ -301,7 +301,7 @@ void DBParser::attribute(string_ref name, string_ref value)
 	}
 }
 
-void DBParser::text(string_ref txt)
+void DBParser::text(string_view txt)
 {
 	if (unknownLevel) return;
 
@@ -325,10 +325,10 @@ void DBParser::text(string_ref txt)
 		try {
 			genMSXid = fast_stou(txt);
 		} catch (std::invalid_argument&) {
-			cliComm.printWarning(StringOp::Builder() <<
+			cliComm.printWarning(
 				"Ignoring bad Generation MSX id (genmsxid) "
-				"in entry with title '" << title <<
-				": " << txt);
+				"in entry with title '", title,
+				": ", txt);
 		}
 		break;
 	case ORIGINAL:
@@ -359,7 +359,7 @@ void DBParser::text(string_ref txt)
 	}
 }
 
-String32 DBParser::cIndex(string_ref str)
+String32 DBParser::cIndex(string_view str)
 {
 	auto* begin = const_cast<char*>(str.data());
 	auto* end = begin + str.size();
@@ -372,11 +372,6 @@ String32 DBParser::cIndex(string_ref str)
 // called on </software>
 void DBParser::addEntries()
 {
-	if (!system.empty() && !small_compare<'M','S','X'>(system)) {
-		// skip non-MSX entries
-		return;
-	}
-
 	for (auto& d : dumps) {
 		db.emplace_back(d.hash, RomInfo(
 			title, year, company, country,
@@ -413,7 +408,7 @@ void DBParser::addAllEntries()
 	while (it2 != last) {
 		if (it1->first == it2->first) {
 			cliComm.printWarning(
-				"duplicate softwaredb entry SHA1: " +
+				"duplicate softwaredb entry SHA1: ",
 				it2->first.toString());
 		} else {
 			++it1;
@@ -457,7 +452,7 @@ void DBParser::addAllEntries()
 	swap(result, db);
 }
 
-static const char* parseStart(string_ref s)
+static const char* parseStart(string_view s)
 {
 	// we expect "0x0000", "0x4000", "0x8000", "0xc000" or ""
 	return ((s.size() == 6) && s.starts_with("0x")) ? (s.data() + 2) : nullptr;
@@ -498,19 +493,19 @@ void DBParser::stop()
 		state = DUMP;
 		break;
 	case ROM: {
-		string_ref t = type;
+		string_view t = type;
 		char buf[12];
 		if (small_compare<'M','i','r','r','o','r','e','d'>(t)) {
 			if (const char* s = parseStart(startVal)) {
 				memcpy(buf, t.data(), 8);
 				memcpy(buf + 8, s, 4);
-				t = string_ref(buf, 12);
+				t = string_view(buf, 12);
 			}
 		} else if (small_compare<'N','o','r','m','a','l'>(t)) {
 			if (const char* s = parseStart(startVal)) {
 				memcpy(buf, t.data(), 6);
 				memcpy(buf + 6, s, 4);
-				t = string_ref(buf, 10);
+				t = string_view(buf, 10);
 			}
 		}
 		RomType romType = RomInfo::nameToRomType(t);
@@ -539,13 +534,13 @@ void DBParser::stop()
 	}
 }
 
-void DBParser::doctype(string_ref txt)
+void DBParser::doctype(string_view txt)
 {
 	auto pos1 = txt.find(" SYSTEM \"");
-	if (pos1 == string_ref::npos) return;
+	if (pos1 == string_view::npos) return;
 	auto t = txt.substr(pos1 + 9);
 	auto pos2 = t.find('"');
-	if (pos2 == string_ref::npos) return;
+	if (pos2 == string_view::npos) return;
 	systemID = t.substr(0, pos2);
 }
 
@@ -563,8 +558,7 @@ static void parseDB(CliComm& cliComm, char* buf, char* bufStart,
 	}
 }
 
-RomDatabase::RomDatabase(GlobalCommandController& commandController, CliComm& cliComm)
-	: softwareInfoTopic(commandController.getOpenMSXInfoCommand())
+RomDatabase::RomDatabase(CliComm& cliComm)
 {
 	db.reserve(3500);
 	UnknownTypes unknownTypes;
@@ -595,8 +589,8 @@ RomDatabase::RomDatabase(GlobalCommandController& commandController, CliComm& cl
 
 			parseDB(cliComm, buf, buffer.data(), db, unknownTypes);
 		} catch (rapidsax::ParseError& e) {
-			cliComm.printWarning(StringOp::Builder() <<
-				"Rom database parsing failed: " << e.what());
+			cliComm.printWarning(
+				"Rom database parsing failed: ", e.what());
 		} catch (MSXException& /*e*/) {
 			// Ignore, see above
 		}
@@ -608,10 +602,9 @@ RomDatabase::RomDatabase(GlobalCommandController& commandController, CliComm& cl
 			"This may cause incorrect ROM mapper types to be used.");
 	}
 	if (!unknownTypes.empty()) {
-		StringOp::Builder output;
-		output << "Unknown mapper types in software database: ";
+		string output = "Unknown mapper types in software database: ";
 		for (auto& p : unknownTypes) {
-			output << p.first << " (" << p.second << "x); ";
+			strAppend(output, p.first, " (", p.second, "x); ");
 		}
 		cliComm.printWarning(output);
 	}
@@ -625,61 +618,4 @@ const RomInfo* RomDatabase::fetchRomInfo(const Sha1Sum& sha1sum) const
 		? &it->second : nullptr;
 }
 
-
-// SoftwareInfoTopic
-
-RomDatabase::SoftwareInfoTopic::SoftwareInfoTopic(
-		InfoCommand& openMSXInfoCommand)
-	: InfoTopic(openMSXInfoCommand, "software")
-{
-}
-
-void RomDatabase::SoftwareInfoTopic::execute(
-	array_ref<TclObject> tokens, TclObject& result) const
-{
-	if (tokens.size() != 3) {
-		throw CommandException("Wrong number of parameters");
-	}
-
-	Sha1Sum sha1sum = Sha1Sum(tokens[2].getString());
-	auto& romDatabase = OUTER(RomDatabase, softwareInfoTopic);
-	const RomInfo* romInfo = romDatabase.fetchRomInfo(sha1sum);
-	if (!romInfo) {
-		// no match found
-		throw CommandException(
-			"Software with sha1sum " + sha1sum.toString() + " not found");
-	}
-
-	const char* bufStart = romDatabase.buffer.data();
-	result.addListElement("title");
-	result.addListElement(romInfo->getTitle(bufStart));
-	result.addListElement("year");
-	result.addListElement(romInfo->getYear(bufStart));
-	result.addListElement("company");
-	result.addListElement(romInfo->getCompany(bufStart));
-	result.addListElement("country");
-	result.addListElement(romInfo->getCountry(bufStart));
-	result.addListElement("orig_type");
-	result.addListElement(romInfo->getOrigType(bufStart));
-	result.addListElement("remark");
-	result.addListElement(romInfo->getRemark(bufStart));
-	result.addListElement("original");
-	result.addListElement(romInfo->getOriginal());
-	result.addListElement("mapper_type_name");
-	result.addListElement(RomInfo::romTypeToName(romInfo->getRomType()));
-	result.addListElement("genmsxid");
-	result.addListElement(romInfo->getGenMSXid());
-}
-
-string RomDatabase::SoftwareInfoTopic::help(const vector<string>& /*tokens*/) const
-{
-	return "Returns information about the software "
-	       "given its sha1sum, in a paired list.";
-}
-
-void RomDatabase::SoftwareInfoTopic::tabCompletion(vector<string>& /*tokens*/) const
-{
-	// no useful completion possible
-}
-
 } // namespace openmsx
