@@ -1,5 +1,4 @@
 #include "RomSuperLodeRunner.hh"
-#include "MSXCPUInterface.hh"
 #include "serialize.hh"
 #include <cassert>
 
@@ -9,15 +8,9 @@ RomSuperLodeRunner::RomSuperLodeRunner(
 		const DeviceConfig& config, Rom&& rom_)
 	: Rom16kBBlocks(config, std::move(rom_))
 {
-	getCPUInterface().registerGlobalWrite(*this, 0x0000);
 	reset(EmuTime::dummy());
 }
 
-RomSuperLodeRunner::~RomSuperLodeRunner()
-{
-	getCPUInterface().unregisterGlobalWrite(*this, 0x0000);
-}
-
 void RomSuperLodeRunner::reset(EmuTime::param /*time*/)
 {
 	setUnmapped(0);
