@@ -3,14 +3,14 @@
 #include "DummyMidiInDevice.hh"
 #include "checked_cast.hh"
 #include "serialize.hh"
-#include "memory.hh"
+#include <memory>
 
 namespace openmsx {
 
 MidiInConnector::MidiInConnector(PluggingController& pluggingController_,
                                  std::string name_)
 	: Connector(pluggingController_, std::move(name_),
-	            make_unique<DummyMidiInDevice>())
+	            std::make_unique<DummyMidiInDevice>())
 {
 }
 
@@ -19,7 +19,7 @@ const std::string MidiInConnector::getDescription() const
 	return "MIDI-in connector";
 }
 
-string_ref MidiInConnector::getClass() const
+string_view MidiInConnector::getClass() const
 {
 	return "midi in";
 }
