@@ -4,7 +4,6 @@
 #include "EventDistributor.hh"
 #include "Scheduler.hh"
 #include "FileOperations.hh"
-#include "StringOp.hh"
 #include "serialize.hh"
 #include <cstdio>
 #include <cerrno>
@@ -36,8 +35,7 @@ void MidiInReader::plugHelper(Connector& connector_, EmuTime::param /*time*/)
 {
 	file = FileOperations::openFile(readFilenameSetting.getString().str(), "rb");
 	if (!file) {
-		throw PlugException(StringOp::Builder()
-			<< "Failed to open input: " << strerror(errno));
+		throw PlugException("Failed to open input: ", strerror(errno));
 	}
 
 	auto& midiConnector = static_cast<MidiInConnector&>(connector_);
@@ -63,7 +61,7 @@ const string& MidiInReader::getName() const
 	return name;
 }
 
-string_ref MidiInReader::getDescription() const
+string_view MidiInReader::getDescription() const
 {
 	return "MIDI in file reader. Sends data from an input file to the "
 	       "MIDI port it is connected to. The filename is set with "
