@@ -4,11 +4,12 @@
 #include "PluggingController.hh"
 #include "PlugException.hh"
 #include "serialize.hh"
-#include "memory.hh"
 #include "openmsx.hh"
+#include "StringOp.hh"
 
 #include <mach/mach_time.h>
 #include <cassert>
+#include <memory>
 
 
 namespace openmsx {
@@ -44,7 +45,7 @@ void MidiOutCoreMIDI::registerAll(PluggingController& controller)
 		MIDIEndpointRef endpoint = MIDIGetDestination(i);
 		if (endpoint) {
 			controller.registerPluggable(
-				make_unique<MidiOutCoreMIDI>(endpoint));
+				std::make_unique<MidiOutCoreMIDI>(endpoint));
 		}
 	}
 }
@@ -63,8 +64,7 @@ MidiOutCoreMIDI::MidiOutCoreMIDI(MIDIEndpointRef endpoint_)
 	if (status) {
 		name = "Nameless endpoint";
 	} else {
-		name = StringOp::Builder() << StringOp::fromCFString(midiDeviceName)
-		                           << " OUT";
+		name = strCat(StringOp::fromCFString(midiDeviceName), " OUT");
 		CFRelease(midiDeviceName);
 	}
 }
@@ -74,15 +74,13 @@ void MidiOutCoreMIDI::plugHelper(Connector& /*connector*/,
 {
 	// Create client.
 	if (OSStatus status = MIDIClientCreate(CFSTR("openMSX"), nullptr, nullptr, &client)) {
-		throw PlugException(StringOp::Builder() <<
-			"Failed to create MIDI client (" << status << ")");
+		throw PlugException("Failed to create MIDI client (", status, ')');
 	}
 	// Create output port.
 	if (OSStatus status = MIDIOutputPortCreate(client, CFSTR("Output"), &port)) {
 		MIDIClientDispose(client);
 		client = 0;
-		throw PlugException(StringOp::Builder() <<
-			"Failed to create MIDI port (" << status << ")");
+		throw PlugException("Failed to create MIDI port (", status, ')');
 	}
 }
 
@@ -103,7 +101,7 @@ const std::string& MidiOutCoreMIDI::getName() const
 	return name;
 }
 
-string_ref MidiOutCoreMIDI::getDescription() const
+string_view MidiOutCoreMIDI::getDescription() const
 {
 	return "Sends MIDI events to an existing CoreMIDI destination.";
 }
@@ -134,14 +132,12 @@ void MidiOutCoreMIDIVirtual::plugHelper(Connector& /*connector*/,
 {
 	// Create client.
 	if (OSStatus status = MIDIClientCreate(CFSTR("openMSX"), nullptr, nullptr, &client)) {
-		throw PlugException(StringOp::Builder() <<
-			"Failed to create MIDI client (" << status << ")");
+		throw PlugException("Failed to create MIDI client (", status, ')');
 	}
 	// Create endpoint.
 	if (OSStatus status = MIDISourceCreate(client, CFSTR("openMSX"), &endpoint)) {
 		MIDIClientDispose(client);
-		throw PlugException(StringOp::Builder() <<
-			"Failed to create MIDI endpoint (" << status << ")");
+		throw PlugException("Failed to create MIDI endpoint (", status, ')');
 	}
 }
 
@@ -165,7 +161,7 @@ const std::string& MidiOutCoreMIDIVirtual::getName() const
 	return name;
 }
 
-string_ref MidiOutCoreMIDIVirtual::getDescription() const
+string_view MidiOutCoreMIDIVirtual::getDescription() const
 {
 	return "Sends MIDI events from a newly created CoreMIDI virtual source.";
 }
