@@ -37,7 +37,7 @@ void ProxySetting::setValue(const TclObject& value)
 	}
 }
 
-string_ref ProxySetting::getTypeString() const
+string_view ProxySetting::getTypeString() const
 {
 	if (auto* setting = getSetting()) {
 		return setting->getTypeString();
@@ -46,7 +46,7 @@ string_ref ProxySetting::getTypeString() const
 	}
 }
 
-string_ref ProxySetting::getDescription() const
+string_view ProxySetting::getDescription() const
 {
 	if (auto* setting = getSetting()) {
 		return setting->getDescription();
@@ -60,7 +60,7 @@ const TclObject& ProxySetting::getValue() const
 	if (auto* setting = getSetting()) {
 		return setting->getValue();
 	} else {
-		throw MSXException("No setting '" + getFullName() + "' on current machine.");
+		throw MSXException("No setting '", getFullName(), "' on current machine.");
 	}
 }
 
@@ -88,7 +88,7 @@ void ProxySetting::setValueDirect(const TclObject& value)
 		// note: not setStringDirect()
 		setting->setValue(value);
 	} else {
-		throw MSXException("No setting '" + getFullName() + "' on current machine.");
+		throw MSXException("No setting '", getFullName(), "' on current machine.");
 	}
 }
 
