@@ -2,9 +2,9 @@
 
 namespace openmsx {
 
-string_ref DummyAudioInputDevice::getDescription() const
+string_view DummyAudioInputDevice::getDescription() const
 {
-	return "";
+	return {};
 }
 
 void DummyAudioInputDevice::plugHelper(Connector& /*connector*/,
