@@ -12,9 +12,9 @@ byte DummyY8950KeyboardDevice::read(EmuTime::param /*time*/)
 	return 255;
 }
 
-string_ref DummyY8950KeyboardDevice::getDescription() const
+string_view DummyY8950KeyboardDevice::getDescription() const
 {
-	return "";
+	return {};
 }
 
 void DummyY8950KeyboardDevice::plugHelper(Connector& /*connector*/,
