@@ -540,7 +540,7 @@ void ResampleHQ<CHANNELS>::calcOutput(
 	bufIdx *= CHANNELS;
 	const float* buf = &buffer[bufIdx];
 
-	int t = unsigned(int(pos * TAB_LEN + 0.5f)) % TAB_LEN;
+	int t = unsigned(lrintf(pos * TAB_LEN)) % TAB_LEN;
 	if (!(t & HALF_TAB_LEN)) {
 		// first half, begin of row 't'
 		t = permute[t];
@@ -567,7 +567,7 @@ void ResampleHQ<CHANNELS>::calcOutput(
 				r2 += tab[i + 2] * buf[CHANNELS * (i + 2)];
 				r3 += tab[i + 3] * buf[CHANNELS * (i + 3)];
 			}
-			output[ch] = lrint(r0 + r1 + r2 + r3);
+			output[ch] = lrintf(r0 + r1 + r2 + r3);
 			++buf;
 		}
 	} else {
@@ -596,7 +596,7 @@ void ResampleHQ<CHANNELS>::calcOutput(
 				r2 += tab[-i - 3] * buf[CHANNELS * (i + 2)];
 				r3 += tab[-i - 4] * buf[CHANNELS * (i + 3)];
 			}
-			output[ch] = lrint(r0 + r1 + r2 + r3);
+			output[ch] = lrintf(r0 + r1 + r2 + r3);
 			++buf;
 		}
 	}
