@@ -6,7 +6,6 @@
 #include "ThrottleManager.hh"
 #include "MSXException.hh"
 #include "Math.hh"
-#include "StringOp.hh"
 #include "Timer.hh"
 #include "build-info.hh"
 #include <SDL.h>
@@ -30,15 +29,14 @@ SDLSoundDriver::SDLSoundDriver(Reactor& reactor_,
 	desired.userdata = this;
 
 	if (SDL_InitSubSystem(SDL_INIT_AUDIO) != 0) {
-		throw MSXException(StringOp::Builder()
-			<< "Unable to initialize SDL audio subsystem: "
-			<< SDL_GetError());
+		throw MSXException(
+			"Unable to initialize SDL audio subsystem: ",
+			SDL_GetError());
 	}
 	SDL_AudioSpec audioSpec;
 	if (SDL_OpenAudio(&desired, &audioSpec) != 0) {
 		SDL_QuitSubSystem(SDL_INIT_AUDIO);
-		throw MSXException(StringOp::Builder()
-			<< "Unable to open SDL audio: " << SDL_GetError());
+		throw MSXException("Unable to open SDL audio: ", SDL_GetError());
 	}
 
 	frequency = audioSpec.freq;
