@@ -10,8 +10,8 @@
 #include "MSXException.hh"
 #include "likely.hh"
 #include "vla.hh"
-#include "memory.hh"
 #include <cassert>
+#include <memory>
 
 using std::string;
 
@@ -28,20 +28,33 @@ static void allocateMixBuffer(unsigned size)
 	}
 }
 
-static string makeUnique(MSXMixer& mixer, string_ref name)
+static string makeUnique(MSXMixer& mixer, string_view name)
 {
 	string result = name.str();
 	if (mixer.findDevice(result)) {
 		unsigned n = 0;
 		do {
-			result = StringOp::Builder() << name << " (" << ++n << ')';
+			result = strCat(name, " (", ++n, ')');
 		} while (mixer.findDevice(result));
 	}
 	return result;
 }
 
-SoundDevice::SoundDevice(MSXMixer& mixer_, string_ref name_,
-			 string_ref description_,
+void SoundDevice::addFill(int*& buf, int val, unsigned num)
+{
+	// Note: in the past we tried to optimize this by always producing
+	// a multiple of 4 output values. In the general case a sounddevice is
+	// allowed to do this, but only at the end of the soundbuffer. This
+	// method can also be called in the middle of a buffer (so multiple
+	// times per buffer), in such case it does go wrong.
+	assert(num > 0);
+	do {
+		*buf++ += val;
+	} while (--num);
+}
+
+SoundDevice::SoundDevice(MSXMixer& mixer_, string_view name_,
+			 string_view description_,
 			 unsigned numChannels_, bool stereo_)
 	: mixer(mixer_)
 	, name(makeUnique(mixer, name_))
@@ -68,7 +81,7 @@ bool SoundDevice::isStereo() const
 	return stereo == 2 || !balanceCenter;
 }
 
-int SoundDevice::getAmplificationFactor() const
+int SoundDevice::getAmplificationFactorImpl() const
 {
 	return 1;
 }
@@ -78,7 +91,7 @@ void SoundDevice::registerSound(const DeviceConfig& config)
 	const XMLElement& soundConfig = config.getChild("sound");
 	float volume = soundConfig.getChildDataAsInt("volume") / 32767.0f;
 	int devBalance = 0;
-	string_ref mode = soundConfig.getChildData("mode", "mono");
+	string_view mode = soundConfig.getChildData("mode", "mono");
 	if (mode == "mono") {
 		devBalance = 0;
 	} else if (mode == "left") {
@@ -86,7 +99,7 @@ void SoundDevice::registerSound(const DeviceConfig& config)
 	} else if (mode == "right") {
 		devBalance = 100;
 	} else {
-		throw MSXException("balance \"" + mode + "\" illegal");
+		throw MSXException("balance \"", mode, "\" illegal");
 	}
 
 	for (auto& b : soundConfig.getChildren("balance")) {
@@ -99,8 +112,7 @@ void SoundDevice::registerSound(const DeviceConfig& config)
 
 		// TODO Support other balances
 		if (balance != 0 && balance != -100 && balance != 100) {
-			throw MSXException(StringOp::Builder() <<
-			                   "balance " << balance << " illegal");
+			throw MSXException("balance ", balance, " illegal");
 		}
 		if (balance != 0) {
 			balanceCenter = false;
@@ -125,12 +137,25 @@ void SoundDevice::updateStream(EmuTime::param time)
 	mixer.updateStream(time);
 }
 
+void SoundDevice::setSoftwareVolume(VolumeType volume, EmuTime::param time)
+{
+	setSoftwareVolume(volume, volume, time);
+}
+
+void SoundDevice::setSoftwareVolume(VolumeType left, VolumeType right, EmuTime::param time)
+{
+	updateStream(time);
+	softwareVolumeLeft  = left;
+	softwareVolumeRight = right;
+	mixer.updateSoftwareVolume(*this);
+}
+
 void SoundDevice::recordChannel(unsigned channel, const Filename& filename)
 {
 	assert(channel < numChannels);
 	bool wasRecording = writer[channel] != nullptr;
 	if (!filename.empty()) {
-		writer[channel] = make_unique<Wav16Writer>(
+		writer[channel] = std::make_unique<Wav16Writer>(
 			filename, stereo, inputSampleRate);
 	} else {
 		writer[channel].reset();
@@ -223,8 +248,11 @@ bool SoundDevice::mixChannels(int* dataOut, unsigned samples)
 		if (writer[i]) {
 			assert(bufs[i] != dataOut);
 			if (bufs[i]) {
+				auto amp = getAmplificationFactor();
 				writer[i]->write(
-					bufs[i], stereo, samples, getAmplificationFactor());
+					bufs[i], stereo, samples,
+					amp.first.toFloat(),
+					amp.second.toFloat());
 			} else {
 				writer[i]->writeSilence(stereo, samples);
 			}
