@@ -2,7 +2,7 @@
 
 namespace openmsx {
 
-string_ref Y8950KeyboardDevice::getClass() const
+string_view Y8950KeyboardDevice::getClass() const
 {
 	return "Y8950 Keyboard Port";
 }
