@@ -3,8 +3,9 @@
 #include "YM2413Burczynski.hh"
 #include "DeviceConfig.hh"
 #include "serialize.hh"
-#include "memory.hh"
 #include "outer.hh"
+#include <cmath>
+#include <memory>
 
 namespace openmsx {
 
@@ -34,9 +35,9 @@ void YM2413::Debuggable::write(unsigned address, byte value, EmuTime::param time
 static std::unique_ptr<YM2413Core> createCore(const DeviceConfig& config)
 {
 	if (config.getChildDataAsBool("alternative", false)) {
-		return make_unique<YM2413Burczynski::YM2413>();
+		return std::make_unique<YM2413Burczynski::YM2413>();
 	} else {
-		return make_unique<YM2413Okazaki::YM2413>();
+		return std::make_unique<YM2413Okazaki::YM2413>();
 	}
 }
 
@@ -46,7 +47,7 @@ YM2413::YM2413(const std::string& name_, const DeviceConfig& config)
 	, debuggable(config.getMotherBoard(), getName())
 {
 	float input = YM2413Core::CLOCK_FREQ / 72.0f;
-	setInputRate(int(input + 0.5f));
+	setInputRate(lrintf(input));
 
 	registerSound(config);
 }
@@ -73,7 +74,7 @@ void YM2413::generateChannels(int** bufs, unsigned num)
 	core->generateChannels(bufs, num);
 }
 
-int YM2413::getAmplificationFactor() const
+int YM2413::getAmplificationFactorImpl() const
 {
 	return core->getAmplificationFactor();
 }
