@@ -2,7 +2,6 @@
 
 #include "win32-windowhandle.hh"
 #include "MSXException.hh"
-#include "StringOp.hh"
 #include <SDL.h>
 #ifdef WIN32_LEAN_AND_MEAN
 #undef WIN32_LEAN_AND_MEAN // For <SDL_syswm.h>, which defines it carelessly
@@ -21,8 +20,7 @@ HWND getWindowHandle()
 	SDL_SysWMinfo info;
 	SDL_VERSION(&info.version);
 	if (SDL_GetWMInfo(&info) != 1) {
-		throw MSXException(StringOp::Builder() <<
-			"SDL_GetWMInfo failed: " << SDL_GetError());
+		throw MSXException("SDL_GetWMInfo failed: ", SDL_GetError());
 	}
 	return info.window;
 }
