@@ -5,7 +5,6 @@
 #include "build-info.hh"
 #include "Math.hh"
 #include "MemBuffer.hh"
-#include "memory.hh"
 #include "vla.hh"
 #include <SDL.h>
 
@@ -137,6 +136,8 @@ void SDLGLOutputSurface::flushFrameBuffer(unsigned width, unsigned height)
 	glEnableVertexAttribArray(0);
 	glEnableVertexAttribArray(1);
 	glDrawArrays(GL_TRIANGLE_FAN, 0, 4);
+	glDisableVertexAttribArray(1);
+	glDisableVertexAttribArray(0);
 }
 
 void SDLGLOutputSurface::clearScreen()
