@@ -39,7 +39,7 @@ void DirectScalerOutput<Pixel>::releaseLine(unsigned /*y*/, Pixel* /*buf*/)
 template<typename Pixel>
 void DirectScalerOutput<Pixel>::fillLine(unsigned y, Pixel color)
 {
-	Pixel* dstLine = output.getLinePtrDirect<Pixel>(y);
+	auto* dstLine = output.getLinePtrDirect<Pixel>(y);
 	MemoryOps::MemSet<Pixel> memset;
 	memset(dstLine, output.getWidth(), color);
 }
